#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_STATUS    0
static uint8_t wave = 0;
static uint8_t run = 0;

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */
void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRB |= 1<<pin;
    if (val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_STATUS)
    {
        dataBuffer[0] = run;
        usbMsgPtr = dataBuffer;
        return 1;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}


ISR(PCINT0_vect,ISR_BLOCK)
{
    if(wave == 0) 
    {
        out_b(0,1);
        wave = 1;
        run = 1;
    }
    else 
    {
        out_b(0,0);
        wave = 0;
        run = 0;
    }
} 


/* ------------------------------------------------------------------------- */


int main(void)
{

    DDRB  = (1<<PB0)|(1<<PB1); // PB0..PB1 = Output, PB2..PB5 = Input
    PORTB = (1 << PB2);                 // Enable pull-up at PB2
    PCICR |= (1<<PCIE0); // enable PCINT7..0
    PCMSK0 |= (1<<PCINT2); // enable PCINT0 

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();

    for(;;){                /* main event loop */
        usbPoll();
    }

    return 0;
}


/* ------------------------------------------------------------------------- */
